// src/components/VoiceAssistant.tsx
import { useState, useEffect, useRef } from "react";

type Message = {
  role: "user" | "assistant" | "system";
  content: string;
};

export const VoiceAssistant = () => {
  const [input, setInput] = useState("");
  const [messages, setMessages] = useState<Message[]>([]);
  const [isLoading, setIsLoading] = useState(false);
  const currentReplyRef = useRef("");

  useEffect(() => {
    const handleStreamChunk = (_: unknown, chunk: string) => {
      currentReplyRef.current += chunk;
      setMessages(prev => {
        const lastMessage = prev[prev.length - 1];
        if (lastMessage?.role === "assistant") {
          return [
            ...prev.slice(0, -1),
            { role: "assistant", content: currentReplyRef.current }
          ];
        }
        return [...prev, { role: "assistant", content: currentReplyRef.current }];
      });
    };

    const handleStreamEnd = () => {
      currentReplyRef.current = "";
      setIsLoading(false);
    };

    const handleStreamError = (_: unknown, errorMsg: string) => {
      setIsLoading(false);
      setMessages(prev => [
        ...prev,
        { role: "assistant", content: `Error: ${errorMsg}` }
      ]);
    };

    window.electronAPI.onChatStreamChunk(handleStreamChunk);
    window.electronAPI.onChatStreamEnd(handleStreamEnd);
    window.electronAPI.onChatStreamError(handleStreamError);

    return () => {
      window.electronAPI.removeListener("chat-stream-chunk", handleStreamChunk);
      window.electronAPI.removeListener("chat-stream-end", handleStreamEnd);
      window.electronAPI.removeListener("chat-stream-error", handleStreamError);
    };
  }, []);

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    if (!input.trim() || isLoading) return;

    const userMessage: Message = { role: "user", content: input };
    const newMessages = [...messages, userMessage];
    
    setMessages(newMessages);
    setInput("");
    setIsLoading(true);
    currentReplyRef.current = "";

    try {
      await window.electronAPI.chatWithDeepseek(newMessages);
    } catch (error) {
      console.error("Chat error:", error);
      setIsLoading(false);
    }
  };

  return (
    <div className="p-4 border rounded-lg bg-gray-900 border-gray-700 shadow-xl">
      <div className="mb-4 h-64 overflow-y-auto">
        {messages.map((msg, index) => (
          <div
            key={index}
            className={`mb-3 p-3 rounded-lg ${
              msg.role === "user"
                ? "bg-blue-800/30 text-blue-200"
                : "bg-gray-800/50 text-gray-300"
            } backdrop-blur-sm`}
          >
            <strong className="font-semibold text-sm">
              {msg.role === "user" ? "You" : "AI"}:
            </strong>
            <p className="mt-1 text-gray-100 leading-relaxed whitespace-pre-wrap">
              {msg.content}
            </p>
          </div>
        ))}
        {isLoading && (
          <div className="mb-3 p-3 rounded-lg bg-gray-800/50 backdrop-blur-sm">
            <div className="flex items-center text-gray-400">
              <span className="flex-1 font-semibold">Thinking...</span>
              <div className="flex space-x-1">
                <div className="w-2 h-2 bg-blue-500 rounded-full animate-bounce"></div>
                <div className="w-2 h-2 bg-blue-500 rounded-full animate-bounce delay-100"></div>
                <div className="w-2 h-2 bg-blue-500 rounded-full animate-bounce delay-200"></div>
              </div>
            </div>
          </div>
        )}
      </div>
      <form onSubmit={handleSubmit} className="flex gap-3">
        <input
          type="text"
          value={input}
          onChange={(e) => setInput(e.target.value)}
          className="flex-1 px-4 py-2 bg-gray-800 border border-gray-700 rounded-xl 
                  text-gray-200 placeholder-gray-500 focus:outline-none 
                  focus:border-blue-500 focus:ring-2 focus:ring-blue-500/30
                  transition-all duration-200"
          placeholder="输入消息..."
          disabled={isLoading}
        />
        <button
          type="submit"
          className={`px-6 py-2 rounded-xl font-medium transition-all duration-200
                  ${
                    isLoading
                      ? "bg-gray-700 text-gray-500 cursor-not-allowed"
                      : "bg-blue-600 hover:bg-blue-700 text-white"
                  }`}
          disabled={isLoading}
        >
          {isLoading ? "发送中..." : "发送"}
        </button>
      </form>
    </div>
  );
};

export default VoiceAssistant;